import { ModalService } from '@spartacus/storefront';
import { BraintreeUtilsService } from '../utils/braintree-utils.service';
import {
  BraintreePaymentDetails,
  CheckoutData,
  CreditCardPaymentMethod,
} from '../../models/braintree-payment-data.model';
import { Injectable } from '@angular/core';
import * as braintree from 'braintree-web';
import {
  ActiveCartService,
  Address,
  GlobalMessageService,
  GlobalMessageType,
} from '@spartacus/core';
import { BraintreeCheckoutService } from '../checkout/braintree-checkout.service';
import {PageType} from '../../models/braintree-payment-methods.model';

@Injectable({
  providedIn: 'root',
})
export class BraintreeHostedFieldsCheckoutService {

  protected tokenizeHostedFieldsEvent: any;
  protected deviceData: string;
  protected billingAddress: Address;
  protected sameAsShipping: boolean;
  protected pageType: PageType;

  constructor(
    protected braintreeUtils: BraintreeUtilsService,
    protected braintreeCheckoutService: BraintreeCheckoutService,
    protected activeCartService: ActiveCartService,
    protected globalMessageService: GlobalMessageService,
    protected modalService: ModalService
  ) {}

  initializeHostedFields(
    checkoutData: CheckoutData,
    buttonContainer: HTMLElement,
    pageType: PageType
  ): void {
    this.pageType = pageType;

    this.braintreeUtils.createClientInstance(
      checkoutData.configurationData,
      (client: any, deviceData: string) => {
        this.deviceData = deviceData;
        this.createHostedFields(
          client,
          checkoutData.creditCardPaymentMethod,
          buttonContainer
        );
      }
    );
  }

  private createHostedFields(
    client: any,
    creditCardData: CreditCardPaymentMethod,
    buttonContainer: HTMLElement
  ): void {
    braintree.hostedFields.create(
      {
        client,
        styles: {
          // Styling element state
          ':focus': {
            color: 'blue',
          },
          '.valid': {
            color: 'green',
          },
          '.invalid': {
            color: 'red',
          },
        },
        fields: {
          number: {
            selector: '#cardNumber',
          },
          expirationDate: {
            selector: '#expiration-date',
            placeholder: 'MM/YY',
          },
          cvv: {
            selector: '#cvv',
          },
        },
      },
      (hostedFieldsErr, hostedFieldsInstance) => {
        if (hostedFieldsErr) {
          this.braintreeUtils.handleClientError(hostedFieldsErr);
          return;
        }

        // Add a click event listener to PayPal image
        this.tokenizeHostedFieldsEvent = (accountHolderName: string) => {
          // initialize paypal authorization
          hostedFieldsInstance.tokenize((tokenizeErr: any, payload: any) => {
            if (tokenizeErr) {
              this.braintreeUtils.handleClientError(tokenizeErr);
            } else {
              if (
                typeof creditCardData.secure3d !== 'undefined' &&
                creditCardData.secure3d &&
                payload.type === 'CreditCard'
              ) {
                this.verify3DSecure(
                  client,
                  payload,
                  accountHolderName,
                  creditCardData.skip3dSecureLiabilityResult
                );
              } else {
                this.braintreeCheckoutService.processHostedFields(
                  payload,
                  accountHolderName,
                  this.sameAsShipping,
                  this.billingAddress,
                  this.deviceData,
                  this.pageType
                  );
              }
            }
          });
        };
      }
    );
  }

  tokenizeHostedFields(accountHolderName: any, sameAsShipping: boolean, billingAddress: Address): void{
    this.billingAddress = billingAddress;
    this.sameAsShipping = sameAsShipping;
    this.tokenizeHostedFieldsEvent(accountHolderName);
  }

  initialise3dSecure(paymentMethod: BraintreePaymentDetails, checkoutData: CheckoutData, paymentMethodNonce: string): boolean {
    let amount: number;
    this.activeCartService
      .getActive()
      .subscribe((cart) => (amount = cart.totalPrice.value))
      .unsubscribe();

    braintree.client.create(
      {
        authorization: checkoutData.configurationData.client_id,
      },
      (clientErr, clientInstance) => {
        if (clientErr) {
          this.braintreeUtils.handleClientError(clientErr);
          return;
        }
        braintree.threeDSecure.create(
          {
            version: 2,
            client: clientInstance,
          },
          (err, threeDSecure) => {
            threeDSecure.verifyCard(
              {
                onLookupComplete: (data, next) => {
                  console.log(JSON.stringify(data));
                  next();
                },
                amount,
                nonce: paymentMethodNonce,
                addFrame: (err: any, iframe: HTMLIFrameElement) => {
                  const threeDSContainer = document.getElementById(
                    'form-additionals'
                  );
                  threeDSContainer.appendChild(iframe);
                  this.modalService.open(iframe, { centered: true });
                },
                removeFrame: () => {
                  this.modalService.closeActiveModal();
                },
              },
             (error: any, response: any) => {
                if (error) {
                  this.globalMessageService.add(
                    error,
                    GlobalMessageType.MSG_TYPE_ERROR
                  );
                  return;
                } else {
                  // 3DSecure finished, add 3DSecure returned nonce
                  const liabilityShifted = response.liabilityShifted;
                  // allow process card if 3dSecureLiabilityResult is skipped by merchant
                  if (
                    liabilityShifted ||
                    checkoutData.creditCardPaymentMethod.skip3dSecureLiabilityResult
                  ) {
                    // e.currentTarget.submit();
                    this.braintreeCheckoutService.selectBraintreePaymentMethod(paymentMethod.id, response.nonce);
                  } else {
                    this.show3DSecureMessage('error.unsecuredCard');
                  }
                }
              }
            );
          }
        );
      }
    );
    return false;
  }

  private verify3DSecure(
    clientInstance: any,
    paymentResponse: any,
    accountHolderName: string,
    skip3dSecureLiabilityResult: boolean
  ): void {

    let amount: number;
    this.activeCartService
      .getActive()
      .subscribe((cart) => (amount = cart.totalPrice.value))
      .unsubscribe();

    // configurePayPalAlongWithHostedFields();
    // selectPaymentMethod();
    braintree.threeDSecure.create(
      {
        version: 2,
        client: clientInstance,
      },
      (err, threeDSecure) => {
        threeDSecure.verifyCard(
          {
            onLookupComplete: (data, next) => {
              next();
            },
            amount,
            nonce: paymentResponse.nonce,
            addFrame: (err: any, iframe: HTMLIFrameElement) => {
              const threeDSContainer = document.getElementById(
                'form-additionals'
              );
              threeDSContainer.appendChild(iframe);
              this.modalService.open(iframe, { centered: true });
            },
            removeFrame: () => {
              this.modalService.closeActiveModal();
            },
          },
          (error, response) => {
            if (error) {
              this.globalMessageService.add(
                error,
                GlobalMessageType.MSG_TYPE_ERROR
              );
              return;
            } else {
              // 3DSecure finished
              // add 3DSecure returned nonce
              paymentResponse.nonce = response.nonce;
              const liabilityShifted = response.liabilityShifted;
              // allow process card if 3dSecureLiabilityResult is
              // skipped by merchant
              if (liabilityShifted || skip3dSecureLiabilityResult) {
                paymentResponse.liabilityShifted = liabilityShifted;
                this.braintreeCheckoutService.processHostedFields(
                  paymentResponse,
                  accountHolderName,
                  this.sameAsShipping,
                  this.billingAddress,
                  this.deviceData,
                  this.pageType
                  );
              } else {
                this.show3DSecureMessage('error.unsecuredCard');
              }
            }
          }
        );
      }
    );
  }

  private show3DSecureMessage(message): void{
    this.globalMessageService.add(
      {
        key: message,
      },
      GlobalMessageType.MSG_TYPE_ERROR
    );
  }

}
