import {Observable, of} from 'rxjs';
import {catchError, map, mergeMap} from 'rxjs/operators';
import {Actions, Effect, ofType} from '@ngrx/effects';
import {BraintreePaymentDetailsConnector} from '../../connectors/braintree-payment-details.connector';
import {
  CartActions,
  CheckoutActions,
  normalizeHttpError,
  Occ,
  OCC_USER_ID_ANONYMOUS,
  SiteContextActions,
  UserActions,
  withdrawOn
} from '@spartacus/core';
import {BraintreePaymentDetailsActions} from '../actions';
import {Injectable} from '@angular/core';
import PaymentDetails = Occ.PaymentDetails;

@Injectable()
export class BraintreeCheckoutEffect {

  private contextChange$ = this.actions$.pipe(
    ofType(
      SiteContextActions.CURRENCY_CHANGE,
      SiteContextActions.LANGUAGE_CHANGE,
    )
  );


    @Effect()
  createVenmoPaymentDetails$: Observable<
    | UserActions.LoadUserPaymentMethods
    | CheckoutActions.CreatePaymentDetailsSuccess
    | BraintreePaymentDetailsActions.BraintreeCreatePaymentDetailsSuccess
    | CheckoutActions.CreatePaymentDetailsFail> = this.actions$.pipe(
    ofType(BraintreePaymentDetailsActions.CREATE_VENMO_PAYMENT_DETAILS),
    map((action: any) => action.payload),
    mergeMap((payload) => {
      // get information for creating a subscription directly with payment provider
      const venmoPaymentDetails: Observable<PaymentDetails> = this.checkoutPaymentConnector
        .saveVenmoPaymentDetails(payload.userId, payload.selectedAddressCode, payload.venmoPayment, payload.cartId,
          payload.shouldBeSaved, payload.deviceData, payload.fields);
      return this.processPaymentCreationResult(venmoPaymentDetails, payload);
    }),
    withdrawOn(this.contextChange$)
  );

  @Effect()
  createPayPalPaymentDetails$: Observable<
    | UserActions.LoadUserPaymentMethods
    | CheckoutActions.CreatePaymentDetailsSuccess
    | BraintreePaymentDetailsActions.BraintreeCreatePaymentDetailsSuccess
    | CheckoutActions.CreatePaymentDetailsFail> = this.actions$.pipe(
    ofType(BraintreePaymentDetailsActions.CREATE_PAYPAL_PAYMENT_DETAILS),
    map((action: any) => action.payload),
    mergeMap((payload) => {
      // get information for creating a subscription directly with payment provider
      const payPalPaymentDetails: Observable<PaymentDetails> = this.checkoutPaymentConnector
        .savePayPalPaymentDetails(payload.credit, payload.shouldBeSaved, payload.payPalRequest,
          payload.userId, payload.cartId, payload.deviceData, payload.fields);
      return this.processPaymentCreationResult(payPalPaymentDetails, payload);
    }),
    withdrawOn(this.contextChange$)
  );


  @Effect()
  createGooglePayPaymentDetails$: Observable<
    | UserActions.LoadUserPaymentMethods
    | CheckoutActions.CreatePaymentDetailsSuccess
    | BraintreePaymentDetailsActions.BraintreeCreatePaymentDetailsSuccess
    | CheckoutActions.CreatePaymentDetailsFail> = this.actions$.pipe(
    ofType(BraintreePaymentDetailsActions.CREATE_GOOGLE_PAY_PAYMENT_DETAILS),
    map((action: any) => action.payload),
    mergeMap((payload) => {
      // get information for creating a subscription directly with payment provider
      const googlePayPaymentDetails: Observable<PaymentDetails> = this.checkoutPaymentConnector
        .saveGooglePayPaymentDetails(payload.userId, payload.googlePayRequest, payload.shouldBeSaved, payload.cartId,
          payload.deviceData, payload.fields);
      return this.processPaymentCreationResult(googlePayPaymentDetails, payload);
    }),
    withdrawOn(this.contextChange$)
  );


  @Effect()
  createCreditCardPaymentDetails$: Observable<
    | UserActions.LoadUserPaymentMethods
    | CheckoutActions.CreatePaymentDetailsSuccess
    | BraintreePaymentDetailsActions.BraintreeCreatePaymentDetailsSuccess
    | CheckoutActions.CreatePaymentDetailsFail> = this.actions$.pipe(
    ofType(BraintreePaymentDetailsActions.CREATE_CREDIT_CARD_PAYMENT_DETAILS),
    map((action: any) => action.payload),
    mergeMap((payload) => {
      // get information for creating a subscription directly with payment provider
      const creditCardPaymentDetails: Observable<PaymentDetails> = this.checkoutPaymentConnector
        .saveCreditCardPaymentDetails(payload.userId, payload.paymentDetails, payload.selectedAddressCode,
          payload.cartId, payload.shouldBeSaved, payload.deviceData, payload.fields);
      return this.processPaymentCreationResult(creditCardPaymentDetails, payload);
    }),
    withdrawOn(this.contextChange$)
  );


  @Effect()
  createApplePayPaymentDetails$: Observable<
    | UserActions.LoadUserPaymentMethods
    | CheckoutActions.CreatePaymentDetailsSuccess
    | BraintreePaymentDetailsActions.BraintreeCreatePaymentDetailsSuccess
    | CheckoutActions.CreatePaymentDetailsFail> = this.actions$.pipe(
    ofType(BraintreePaymentDetailsActions.CREATE_APPLE_PAY_PAYMENT_DETAILS),
    map((action: any) => action.payload),
    mergeMap((payload) => {
      // get information for creating a subscription directly with payment provider
      const applePayPaymentDetails: Observable<PaymentDetails> = this.checkoutPaymentConnector
        .saveApplePayPaymentDetails(payload.userId, payload.applePayRequest, payload.cartId, payload.deviceData,
          payload.fields);
      return this.processPaymentCreationResult(applePayPaymentDetails, payload);
    }),
    withdrawOn(this.contextChange$)
  );

  @Effect()
  processLocalPaymentMethod: Observable<
    | CheckoutActions.PlaceOrderSuccess
    | CartActions.RemoveCart
    | CheckoutActions.PlaceOrderFail> = this.actions$.pipe(
    ofType(BraintreePaymentDetailsActions.PROCESS_LOCAL_PAYMENT_METHOD),
    map((action: any) => action.payload),
    mergeMap((payload) => {
      // get information for creating a subscription directly with payment provider
  return this.checkoutPaymentConnector
    .saveLocalPaymentMethodPaymentDetails(payload.userId, payload.cartId, payload.localPaymentRequest,
      payload.deviceData, payload.fields) .pipe(mergeMap((data) => {
          return [new CartActions.RemoveCart({ cartId: payload.cartId }),
            new CheckoutActions.PlaceOrderSuccess(data)];
      }),
      catchError((error) =>
        of(new CheckoutActions.PlaceOrderFail(normalizeHttpError(error)))
      )
    );

    }),
    withdrawOn(this.contextChange$)
  );

  @Effect()
  placeOrderThroughFallback$: Observable<
    | CheckoutActions.PlaceOrderSuccess
    | CartActions.RemoveCart
    | CheckoutActions.PlaceOrderFail> = this.actions$.pipe(
      ofType(BraintreePaymentDetailsActions.PLACE_ORDER_THROUGH_FALLBACK),
    map((action: any) => action.payload),
    mergeMap((payload) => {
      return this.checkoutPaymentConnector.processLPMFallback(payload.localPaymentRequest, payload.currencyFromFallbackURL, payload.deviceData, payload.fields)
        .pipe(mergeMap((data) => {
          if (payload.cartId){
            return [new CartActions.RemoveCart({ cartId: payload.cartId }),
                new CheckoutActions.PlaceOrderSuccess(data)];
          } else {
            return [new CheckoutActions.PlaceOrderSuccess(data)];
          }
        }),
      catchError((error) =>
            of(new CheckoutActions.PlaceOrderFail(normalizeHttpError(error)))
          )
        );

    }),
    withdrawOn(this.contextChange$)
  );

  public processPaymentCreationResult(savingResult: Observable<PaymentDetails>, payload: any): Observable<
    | UserActions.LoadUserPaymentMethods
    | CheckoutActions.CreatePaymentDetailsSuccess
    | BraintreePaymentDetailsActions.BraintreeCreatePaymentDetailsSuccess
    | CheckoutActions.CreatePaymentDetailsFail> {

    return savingResult.pipe(mergeMap((details) => {
        if (payload.userId === OCC_USER_ID_ANONYMOUS) {
          return [new BraintreePaymentDetailsActions.BraintreeCreatePaymentDetailsSuccess(details),
          new CheckoutActions.CreatePaymentDetailsSuccess(details)];
        } else {
          return [
            new UserActions.LoadUserPaymentMethods(payload.userId),
            new BraintreePaymentDetailsActions.BraintreeCreatePaymentDetailsSuccess(details),
              new CheckoutActions.CreatePaymentDetailsSuccess(details)
          ];
        }
      }),
      catchError((error) =>
        of(
          new CheckoutActions.CreatePaymentDetailsFail(
            normalizeHttpError(error)
          )
        )
      )
    );

  }

  constructor(
    private actions$: Actions,
    private checkoutPaymentConnector: BraintreePaymentDetailsConnector
  ) {
  }

}
