import {Injectable} from '@angular/core';
import {
  Address,
  CheckoutDeliveryService,
  CheckoutPaymentService,
  DeliveryMode,
  PaymentDetails,
  StateUtils,
  UserAddressService,
  UserPaymentService
} from '@spartacus/core';
import {combineLatest, Observable, of} from 'rxjs';
import {debounceTime, filter, map, switchMap, takeWhile, tap} from 'rxjs/operators';
import {Router} from '@angular/router';
import {CheckoutConfigService, CheckoutDetailsService} from '@spartacus/storefront';
import {BraintreeCheckoutPaymentService} from './braintree-checkout-payment.service';

@Injectable({
  providedIn: 'root',
})
export class BraintreeExpressCheckoutService {
  protected braintreeShippingAddressSet$: Observable<boolean>;
  protected braintreePaymentMethodSet$: Observable<boolean>;
  protected deliveryModeSet$: Observable<boolean>;

  constructor(
    protected userAddressService: UserAddressService,
    protected userPaymentService: UserPaymentService,
    protected checkoutDeliveryService: CheckoutDeliveryService,
    protected checkoutPaymentService: CheckoutPaymentService,
    protected checkoutDetailsService: CheckoutDetailsService,
    protected checkoutConfigService: CheckoutConfigService,
    protected router: Router,
    protected braintreeCheckoutPaymentService: BraintreeCheckoutPaymentService
  ) {}

  processExpressCheckout(
    address: Address,
    paymentDetails: PaymentDetails
  ): void {
    this.setShippingAddress(address);
    this.setDeliveryMode();
    this.setPaymentMethod(paymentDetails);
    this.trySetBraintreeCheckoutDetails()
      .pipe(takeWhile((status) => status !== true, true))
      .subscribe((status) => {
        if (status) {
          this.router.navigate(['/checkout/review-order']);
        }
      });
  }

  protected resetCheckoutProcesses(): void {
    this.checkoutDeliveryService.resetSetDeliveryAddressProcess();
    this.checkoutPaymentService.resetSetPaymentDetailsProcess();
    this.checkoutDeliveryService.resetSetDeliveryModeProcess();
  }

  protected trySetBraintreeCheckoutDetails(): Observable<boolean> {
    this.resetCheckoutProcesses();
    return combineLatest([
      this.deliveryModeSet$,
      this.braintreePaymentMethodSet$,
    ]).pipe(
      map(([deliveryModeSet, paymentMethodSet]) =>
        Boolean(deliveryModeSet && paymentMethodSet)
      )
    );
  }

  protected setPaymentMethod(paymentDetails: PaymentDetails): void {
    this.braintreePaymentMethodSet$ = combineLatest([
      this.deliveryModeSet$,
      this.checkoutPaymentService.getSetPaymentDetailsResultProcess(),
    ]).pipe(
      debounceTime(0),
      switchMap(
        ([deliveryModeSet, setPaymentDetailsProcess]: [
          boolean,
          StateUtils.LoaderState<void>
        ]) => {
            if (
              !(
                setPaymentDetailsProcess.success ||
                setPaymentDetailsProcess.error ||
                setPaymentDetailsProcess.loading
              ) && deliveryModeSet
            ) {
            this.braintreeCheckoutPaymentService.setPaymentDetails(paymentDetails);
          }
          return of(setPaymentDetailsProcess).pipe(
            filter(
              (setPaymentDetailsProcessState: StateUtils.LoaderState<void>) => {
                return (
                  (setPaymentDetailsProcessState.success ||
                    setPaymentDetailsProcessState.error) &&
                  !setPaymentDetailsProcessState.loading
                );
              }
            ),
            switchMap(
              (setPaymentDetailsProcessState: StateUtils.LoaderState<void>) => {
                if (setPaymentDetailsProcessState.success) {
                  return this.checkoutDetailsService.getPaymentDetails();
                }
                return of(false);
              }
            ),
            map((data) => Boolean(data && Object.keys(data).length))
          );
        }
      )
    );
  }

  protected setShippingAddress(address: Address): void {
    this.checkoutDeliveryService.createAndSetAddress(address);
    this.braintreeShippingAddressSet$ = this.checkoutDeliveryService
      .getSetDeliveryAddressProcess()
      .pipe(
        debounceTime(0),
        filter(
          (setDeliveryAddressProcessState: StateUtils.LoaderState<void>) => {
            return (
              (setDeliveryAddressProcessState.success ||
                setDeliveryAddressProcessState.error) &&
              !setDeliveryAddressProcessState.loading
            );
          }
        ),
        switchMap(
          (setDeliveryAddressProcessState: StateUtils.LoaderState<void>) => {
            if (setDeliveryAddressProcessState.success) {
              return this.checkoutDetailsService.getDeliveryAddress();
            }
            return of(false);
          }
        ),
        map((data) => Boolean(data && Object.keys(data).length))
      );
  }

  protected setDeliveryMode(): void {
    this.deliveryModeSet$ = combineLatest([
      this.braintreeShippingAddressSet$,
      this.checkoutDeliveryService.getSupportedDeliveryModes(),
      this.checkoutDeliveryService.getSetDeliveryModeProcess(),
      this.checkoutDeliveryService.getLoadSupportedDeliveryModeProcess(),
    ]).pipe(
      debounceTime(0),
      switchMap(
        ([
          addressSet,
          supportedDeliveryModes,
          setDeliveryModeStatusFlag,
          loadSupportedDeliveryModeStatus,
        ]: [
          boolean,
          DeliveryMode[],
          StateUtils.LoaderState<void>,
          StateUtils.LoaderState<void>
        ]) => {
          if (addressSet) {
            return of([
              supportedDeliveryModes,
              setDeliveryModeStatusFlag,
              loadSupportedDeliveryModeStatus,
            ]).pipe(
              filter(
                ([, , supportedDeliveryModeStatus]: [
                  DeliveryMode[],
                  StateUtils.LoaderState<void>,
                  StateUtils.LoaderState<void>
                ]) => supportedDeliveryModeStatus.success
              ),
              switchMap(
                ([deliveryModes, setDeliveryModeStatus, ,]: [
                  DeliveryMode[],
                  StateUtils.LoaderState<void>,
                  StateUtils.LoaderState<void>
                ]) => {
                  if (Boolean(deliveryModes.length)) {
                    const preferredDeliveryMode =
                      this.checkoutConfigService.getPreferredDeliveryMode(
                        deliveryModes
                      );
                    return of([
                      preferredDeliveryMode,
                      setDeliveryModeStatus,
                    ]).pipe(
                      tap(
                        ([deliveryMode, deliveryModeLoadingStatus]: [
                          string,
                          StateUtils.LoaderState<void>
                        ]) => {
                          if (
                            deliveryMode &&
                            !(
                              deliveryModeLoadingStatus.success ||
                              deliveryModeLoadingStatus.error ||
                              deliveryModeLoadingStatus.loading
                            )
                          ) {
                            this.checkoutDeliveryService.setDeliveryMode(
                              deliveryMode
                            );
                          }
                        }
                      ),
                      filter(
                        ([, deliveryModeLoadingStatus]: [
                          string,
                          StateUtils.LoaderState<void>
                        ]) => {
                          return (
                            (deliveryModeLoadingStatus.success ||
                              deliveryModeLoadingStatus.error) &&
                            !deliveryModeLoadingStatus.loading
                          );
                        }
                      ),
                      switchMap(
                        ([, deliveryModeLoadingStatus]: [
                          string,
                          StateUtils.LoaderState<void>
                        ]) => {
                          if (deliveryModeLoadingStatus.success) {
                            return this.checkoutDetailsService.getSelectedDeliveryModeCode();
                          }
                          return of(false);
                        }
                      ),
                      map((data) => Boolean(data))
                    );
                  }
                  return of(false);
                }
              )
            );
          } else {
            return of(false);
          }
        }
      )
    );
  }
}
