import { BraintreePaymentMethodsErrorHandlerService } from './../errorHandling/braintree-payment-methods-error-handler.service';
import {Injectable} from '@angular/core';
import {BraintreeUtilsService} from '../utils/braintree-utils.service';
import * as CONST from '../../models/braintree.constants';

import * as braintree from 'braintree-web';
import {GlobalMessageService, GlobalMessageType} from '@spartacus/core';
import {PayPalButtonStyle} from '../../models/paypal-button-style.model';
import {PageType} from '../../models/braintree-payment-methods.model';
import {CheckoutData} from '../../models/braintree-payment-data.model';
import {BraintreeCheckoutService} from '../checkout/braintree-checkout.service';

@Injectable({
  providedIn: 'root'
})
export class PaypalCheckoutService {

  protected checkoutData: CheckoutData;
  protected buttonStyle: PayPalButtonStyle;
  protected deviceData: string;
  protected pageType: PageType;

  constructor(
    protected braintreeUtils: BraintreeUtilsService,
    protected globalMessageService: GlobalMessageService,
    protected braintreeCheckoutService: BraintreeCheckoutService,
    protected braintreeErrorHandlerService: BraintreePaymentMethodsErrorHandlerService
  ) { }


  initializePayPalCheckout(buttonContainer, checkoutData: CheckoutData, buttonStyle: PayPalButtonStyle, pageType: PageType): void{
    this.buttonStyle = buttonStyle;
    this.pageType = pageType;

    this.checkoutData = checkoutData;
    let payPalOptions;
    if (PageType.CART === pageType){
      payPalOptions = this.createPayPalOptionsExpressCheckout();
    }else {
      payPalOptions = this.createPayPalOptions(PageType.BILLING);
    }

    this.braintreeUtils.createClientInstance(this.checkoutData.configurationData,  (client, deviceData) => {
      this.deviceData = deviceData;
      this.createPayPalCheckout(payPalOptions, client, buttonContainer, pageType);
    });
  }

  private createPayPalOptionsExpressCheckout(): any {
    // const paypalIntent = this.checkoutData.configurationData.intent;
    const paypalFlow = this.braintreeUtils.getPaypalFlow(this.checkoutData, this.pageType);

    let paypalOptions: {[k: string]: any} = {};
    paypalOptions = {
      flow: paypalFlow,
      enableShippingAddress: true,
      enableBillingAddress: true,
      locale: this.checkoutData.configurationData.braintreeLocale,
    };

    if (paypalFlow === CONST.CHECKOUT_FLOW) {
      this.configurePayPalOptions(paypalOptions, false);
      // configure pay later integration
      if (this.checkoutData.payPalPaymentMethod.creditEnabled && this.braintreeUtils
        .getStoreInVault(this.checkoutData.configurationData.storeInVault)) {
        paypalOptions.requestBillingAgreement = true;
        paypalOptions.billingAgreementDetails = {
          description: this.checkoutData.payPalPaymentMethod.billingAgreementDescription
        };
      }

    }
    else if (paypalFlow === CONST.VAULT_FLOW) {
      paypalOptions.billingAgreementDescription = this.checkoutData.payPalPaymentMethod.billingAgreementDescription;
    }

    const dbaName = this.checkoutData.configurationData.dbaName;
    // configure display name for paypal connection
    if (typeof dbaName !== 'undefined' && dbaName !== '') {
      if (dbaName.indexOf('*') > -1) {
        paypalOptions.displayName = dbaName.substr(0, dbaName.indexOf('*'));
      }
    }
    return paypalOptions;
  }

  private createPayPalCheckout(paypalOptions, client, payPalButtonContainer, pageType: PageType): void {
    if (payPalButtonContainer != null) {
      const commit = paypalOptions.paypalIntent === CONST.INTENT_SALE && paypalOptions.userAction === 'true';
      const isIntentValid = this.braintreeUtils.checkIntentOption(this.checkoutData.configurationData.intent);
      if (isIntentValid === false) {
        this.globalMessageService.add(
          { key: 'intent.notSupported' },
          GlobalMessageType.MSG_TYPE_ERROR);
        return;
      }

      braintree.paypalCheckout.create({
        client
      }, (paypalCheckoutErr, paypalCheckoutInstance) => {

          this.braintreeUtils.loadPayPalSDK(this.checkoutData, paypalCheckoutInstance,
            this.checkoutData.configurationData.intent, commit, paypalOptions.flow, pageType, () => {
            console.log('PayPal Sdk was loaded');
            payPalButtonContainer.innerHTML = '';

            this.payPalCheckoutRenderButton(paypalCheckoutInstance, payPalButtonContainer, paypalOptions, commit);
          });
      });


    }

  }

  private payPalCheckoutRenderButton(paypalCheckoutInstance, payPalButtonContainer, paypalOptions, commit): void {
    const isCheckoutFlow = this.braintreeUtils.getPaypalFlow(this.checkoutData, this.pageType).localeCompare(CONST.CHECKOUT_FLOW);
    if (isCheckoutFlow === 0) {
      if (this.checkoutData.payPalPaymentMethod.creditEnabled
        && this.braintreeUtils.getStoreInVault(this.checkoutData.configurationData.storeInVault)) {
        this.renderCheckoutBillingButtons(paypalCheckoutInstance, payPalButtonContainer, paypalOptions, commit);
      }
      else {
        this.renderCheckoutFlowPayPalButtons(paypalCheckoutInstance, payPalButtonContainer, paypalOptions, commit);
      }
    }
    else {
      this.renderVaultFlowPayPalButtons(paypalCheckoutInstance, payPalButtonContainer, paypalOptions, commit);
    }
  }

  private renderCheckoutFlowPayPalButtons(paypalCheckoutInstance, payPalButtonContainer, paypalOptions, commit): any{
    try {
      (window as any).paypal.Buttons({
        style: this.buttonStyle,
        locale: this.checkoutData.configurationData.braintreeLocale,
        commit,
        onClick: () => {
          paypalOptions.amount = this.braintreeUtils.getTotalAmountBeforeRendering();
        },
        createOrder(): any {
          return paypalCheckoutInstance.createPayment(paypalOptions);
        },
        onApprove: (data) => {
          data.intent = this.checkoutData.configurationData.intent;
          return paypalCheckoutInstance.tokenizePayment(data).then((payload) => {
            this.braintreeCheckoutService.processPayPalResponse(
              payload,
              this.checkoutData.payPalPaymentMethod.creditEnabled,
              this.deviceData,
              this.pageType
            );
          });
        },
        onCancel(): void {
          console.log('User cancel PayPal flow');
        },

        onError: (err) => {
          console.error('Error: ' + err, err);
          this.handlePayPalClientError(err);
        }
      }).render(payPalButtonContainer);
    }
    catch (err) {
      console.log(err.message);
      this.handlePayPalButtonError(err.message);
    }
  }

  private renderVaultFlowPayPalButtons(paypalCheckoutInstance, payPalButtonContainer, paypalOptions, commit): any{
    try {
      (window as any).paypal.Buttons({
        style: this.buttonStyle,
        locale: this.checkoutData.configurationData.braintreeLocale,
        commit,
        onClick: () => {
          paypalOptions.amount = this.braintreeUtils.getTotalAmountBeforeRendering();
        },
        createBillingAgreement(): any {
          return paypalCheckoutInstance.createPayment(paypalOptions);
        },
        onApprove: (data) => {
          return paypalCheckoutInstance.tokenizePayment(data).then((payload) => {
            this.braintreeCheckoutService.processPayPalResponse(
              payload,
              this.checkoutData.payPalPaymentMethod.creditEnabled,
              this.deviceData,
              this.pageType
            );
          });

        },
        onCancel(data): void {
          console.log('User cancel PayPal flow');
        },

        onError: (err) => {
          console.error('Error: ' + err, err);
          this.handlePayPalClientError(err);
        }
      }).render(payPalButtonContainer);
    }
    catch (err) {
      console.log(err.message);
      this.handlePayPalButtonError(err.message);
    }
  }

  private renderCheckoutBillingButtons(paypalCheckoutInstance, payPalButtonContainer, paypalOptions, commit): any{
    try {
      (window as any).paypal.Buttons({
        style: this.buttonStyle,
        locale: this.checkoutData.configurationData.braintreeLocale,
        commit,
        onClick: () => {
          paypalOptions.amount = this.braintreeUtils.getTotalAmountBeforeRendering();
        },
        createOrder(): any {
          return paypalCheckoutInstance.createPayment(paypalOptions);
        },
        onApprove: (data) => {
          return paypalCheckoutInstance.tokenizePayment(data).then((payload) => {
            this.braintreeCheckoutService.processPayPalResponse(
              payload,
              this.checkoutData.payPalPaymentMethod.creditEnabled,
              this.deviceData,
              this.pageType
            );
          });
        },
        onCancel(data): void {
          console.log('User cancel PayPal flow');
        },

        onError: (err) => {
          console.error('Error: ' + err, err);
          this.handlePayPalClientError(err);
        }
      }).render(payPalButtonContainer);
    }
    catch (err) {
      console.log(err.message);
      this.handlePayPalButtonError(err.message);
    }
  }

  private handlePayPalClientError(error): void {
    if (typeof error !== 'undefined' || error !== 'undefined') {
      // skip validation error: use paypal method
      if ('User did not enter a payment method' !== error.message && 'PAYPAL_POPUP_CLOSED' !== error.code) {
        let messageText = error.message;
        if (typeof messageText === 'undefined' || messageText === 'undefined') {
          // Undefined error
        } else {
          this.braintreeErrorHandlerService.getErrorMessage(error.code.toLowerCase()).subscribe(res => {
            if (res !== undefined) {
              messageText = res;
            }
            this.globalMessageService.add(
              {
                key: 'error.provider',
                params: {reason: messageText},
              },
              GlobalMessageType.MSG_TYPE_ERROR
            );
          });
        }
      }
    }
  }

  public createPayPalOptions(buttonType?: PageType): any{
    const paypalFlow = this.braintreeUtils.getPaypalFlow(this.checkoutData, this.pageType);
    const enableShippingAddress: boolean = this.checkoutData.enableShippingAddress;

    const payPalOptions: {[k: string]: any}  = {
      flow: paypalFlow,
      enableShippingAddress: enableShippingAddress,
      enableBillingAddress: true,
      locale: this.checkoutData.configurationData.locale,
      shippingAddressEditable: false
    };

    if (PageType.MY_ACCOUNT === buttonType){
      payPalOptions.billingAgreementDescription = this.checkoutData.payPalPaymentMethod.billingAgreementDescription;
    }else if (CONST.CHECKOUT_FLOW === paypalFlow){

      this.configurePayPalOptions(payPalOptions, enableShippingAddress);

      if (this.checkoutData.payPalPaymentMethod.creditEnabled && this.braintreeUtils
        .getStoreInVault(this.checkoutData.configurationData.storeInVault)) {
        payPalOptions.requestBillingAgreement = true;
        payPalOptions.billingAgreementDetails = {
          description: this.checkoutData.payPalPaymentMethod.billingAgreementDescription
        };
      }

    } else if (CONST.VAULT_FLOW === paypalFlow){

      this.configurePayPalOptions(payPalOptions, enableShippingAddress);

      payPalOptions.billingAgreementDescription = this.checkoutData.payPalPaymentMethod.billingAgreementDescription;

      const dbaName = this.checkoutData.configurationData.dbaName;

      if (typeof dbaName !== undefined && dbaName !== '') {
        if (dbaName.indexOf('*') > -1) {
          payPalOptions.displayName = dbaName.substr(0, dbaName.indexOf('*'));
        }
      }
    }
    return payPalOptions;
  }

  private configurePayPalOptions(payPalOptions, enableShippingAddress): void{
    const paypalIntent = this.checkoutData.configurationData.intent;
    if (paypalIntent !== undefined && paypalIntent !== '') {
      payPalOptions.intent = paypalIntent;
    }
    payPalOptions.currency = this.checkoutData.configurationData.currency;
    // configure pay later integration
    if (enableShippingAddress) {
      payPalOptions.shippingAddressOverride =  payPalOptions.shippingAddressOverride = {
        recipientName: this.checkoutData.shippingAddressOverride.recipientName,
        line1: this.checkoutData.shippingAddressOverride.streetAddress,
        line2: this.checkoutData.shippingAddressOverride.extendedAddress,
        city: this.checkoutData.shippingAddressOverride.locality,
        countryCode: this.checkoutData.shippingAddressOverride.countryCodeAlpha2,
        postalCode: this.checkoutData.shippingAddressOverride.postalCode,
        state: this.checkoutData.shippingAddressOverride.region,
        phone: this.checkoutData.shippingAddressOverride.phone
      };
    }
  }

  private handlePayPalButtonError(errorMsg): void {
    if (typeof errorMsg !== 'undefined' || errorMsg !== 'undefined') {
      this.globalMessageService.add(
        errorMsg,
        GlobalMessageType.MSG_TYPE_ERROR
      );
    }
  }
}
