package com.braintree.facade.backoffice.impl;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.function.Predicate;

import com.braintree.configuration.service.BrainTreeConfigService;
import com.braintree.constants.BraintreeConstants;
import com.braintree.order.capture.partial.services.BraintreePartialCaptureService;
import com.braintree.order.submitForSettlement.service.BraintreeSubmitForSettlementService;
import com.braintreegateway.Transaction;
import de.hybris.platform.core.enums.OrderStatus;
import de.hybris.platform.ordermanagementfacades.payment.data.PaymentTransactionEntryData;
import de.hybris.platform.servicelayer.dto.converter.Converter;
import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;

import com.braintree.exceptions.BraintreeErrorException;
import com.braintree.facade.backoffice.BraintreeBackofficeMultiCaptureFacade;
import com.braintree.method.BrainTreePaymentService;
import com.braintree.model.BrainTreePaymentInfoModel;

import de.hybris.platform.core.model.order.OrderModel;
import de.hybris.platform.payment.dto.TransactionStatus;
import de.hybris.platform.payment.enums.PaymentTransactionType;
import de.hybris.platform.payment.model.PaymentTransactionEntryModel;
import de.hybris.platform.payment.model.PaymentTransactionModel;
import de.hybris.platform.servicelayer.model.ModelService;
import org.springframework.beans.factory.annotation.Qualifier;

import javax.annotation.Resource;


public class BraintreeBackofficeMultiCaptureFacadeImpl implements BraintreeBackofficeMultiCaptureFacade {

    private final static Logger LOG = Logger.getLogger(BraintreeBackofficeMultiCaptureFacadeImpl.class);

    @Autowired
    private BrainTreePaymentService brainTreePaymentService;
    @Autowired
    private ModelService modelService;
    @Autowired
    private BrainTreeConfigService brainTreeConfigService;
    @Autowired
    @Qualifier("braintreePartialCaptureService")
    private BraintreePartialCaptureService braintreePartialCaptureService;
    @Autowired
    private BraintreeSubmitForSettlementService braintreeSubmitForSettlementService;
    @Autowired
    @Qualifier("defaultPaymentTransactionEntryConverter")
    private Converter<PaymentTransactionEntryModel, PaymentTransactionEntryData> ordermanagementPaymentTransactionEntryConverter;

    @Override
    public boolean isMultiCapturePossible(final OrderModel order) {
        if (null == order) {
            LOG.warn("order: " + order);
            return false;
        }
        LOG.info("isMultiCapturePossible, order.getTotalPrice: " + order.getTotalPrice());

        boolean isMultiCapturePossible = false;
        modelService.refresh(order);
        Boolean settlementConfigParameter = brainTreeConfigService.getSettlementConfigParameter();

        if (order.getPaymentInfo() instanceof BrainTreePaymentInfoModel) {
            final String intent = getOrderIntent(order);

            if (paidByCard(order) || isVenmoPayment(order)) {
                if (!settlementConfigParameter && !isOrderVoided(order) && !(OrderStatus.COMPLETED
                    .equals(order.getStatus()))) {
                    isMultiCapturePossible = true;
                }
            } else if (!isOrderVoided(order) && !(OrderStatus.COMPLETED.equals(order.getStatus()))) {
                isMultiCapturePossible = isSuccessfulTransactionPresent(order, PaymentTransactionType.AUTHORIZATION) &&
                    !BraintreeConstants.PAYPAL_INTENT_SALE.equalsIgnoreCase(intent);
            }
        }
        return isMultiCapturePossible;
    }

    private String getOrderIntent(final OrderModel order) {
        return ((BrainTreePaymentInfoModel) order.getPaymentInfo()).getPayPalIntent();
    }

    @Override
    public List<PaymentTransactionEntryModel> getMultiCaptureableTransactions(final OrderModel order) {
        List<PaymentTransactionEntryModel> result = new ArrayList<>();

        for (PaymentTransactionModel transaction : order.getPaymentTransactions()) {
            for (PaymentTransactionEntryModel paymentEntry : transaction.getEntries()) {
                if (PaymentTransactionType.AUTHORIZATION.equals(paymentEntry.getType())
                    && TransactionStatus.ACCEPTED.name().equals(paymentEntry.getTransactionStatus())) {
                    result.add(paymentEntry);
                }
            }
        }

        return result;
    }

    @Override
    public void executeMultiCapture(final PaymentTransactionEntryModel transaction) throws BraintreeErrorException {
        LOG.warn("brainTreePaymentService: " + brainTreePaymentService);
    }

    private boolean isSuccessfulTransactionPresent(final OrderModel order,
        final PaymentTransactionType transactionType) {
        for (PaymentTransactionModel paymentTransaction : order.getPaymentTransactions()) {
            for (PaymentTransactionEntryModel transactionEntry : paymentTransaction.getEntries()) {
                if ((transactionType.equals(transactionEntry.getType()) && transactionEntry
                    .getTransactionStatusDetails().startsWith("SUCCESFULL"))
                    && TransactionStatus.ACCEPTED.name().equals(transactionEntry.getTransactionStatus())) {
                    return true;
                }
            }
        }
        return false;
    }

    @Override
    public PaymentTransactionEntryData partialCapture(OrderModel orderModel, BigDecimal captureAmount,
        final String authorizeTransactionID) throws BraintreeErrorException {
        return ordermanagementPaymentTransactionEntryConverter
            .convert(braintreePartialCaptureService.partialCapture(orderModel, captureAmount, authorizeTransactionID));
    }

    @Override
    public BigDecimal getPossibleAmountForCapture(OrderModel orderModel) {
        return braintreePartialCaptureService.getPossibleAmountForCapture(orderModel);
    }

    @Override
    public boolean isPartialCapturePossible(OrderModel orderModel) {
        return braintreePartialCaptureService.isPartialCapturePossible(orderModel);
    }

    public BigDecimal getTransactionAmount(final PaymentTransactionEntryModel currentModel,
        final OrderModel orderModel) {
        return braintreePartialCaptureService.getTransactionAmount(currentModel, orderModel);
    }

    @Override
    public PaymentTransactionEntryData submitForSettlement(OrderModel orderModel, BigDecimal amount,
        String authorizeTransactionID)
        throws BraintreeErrorException {
        return ordermanagementPaymentTransactionEntryConverter
            .convert(
                braintreeSubmitForSettlementService.submitForSettlement(orderModel, amount, authorizeTransactionID));
    }

    @Override
    public boolean isSubmitForSettlementAvailable(OrderModel orderModel) {
        return braintreeSubmitForSettlementService.isSubmitForSettlementAvailable(orderModel);
    }

    @Override
    public Predicate<PaymentTransactionEntryModel> getCaptureTransactionTypePredicate() {
        return braintreePartialCaptureService.getCaptureTransactionTypePredicate();
    }

    @Override
    public Predicate<PaymentTransactionEntryModel> getAuthorizeTransactionTypePredicate() {
        return braintreePartialCaptureService.getAuthorizeTransactionTypePredicate();
    }

    @Override
    public Predicate<PaymentTransactionEntryModel> getRefundTransactionTypePredicate() {
        return braintreePartialCaptureService.getRefundTransactionTypePredicate();
    }

    private boolean isOrderVoided(final OrderModel order) {
        boolean isOrderVoided = false;
        PaymentTransactionEntryModel firstEntry = order.getPaymentTransactions().stream()
            .flatMap(transaction -> transaction.getEntries()
                .stream()).findFirst().orElse(null);
        if (firstEntry != null && Transaction.Status.VOIDED.name().equals(firstEntry.getTransactionStatus())) {
            isOrderVoided = true;
        }
        return isOrderVoided;
    }

    private boolean paidByCard(final OrderModel order) {
        boolean paidByCard = false;
        Object cardType = order.getPaymentInfo().getProperty("cardType");
        if (cardType != null) {
            paidByCard = true;
        }
        return paidByCard;
    }

    private boolean isVenmoPayment(final OrderModel order) {
        return order.getPaymentInfo().getProperty("paymentProvider").equals(BraintreeConstants.VENMO_CHECKOUT);
    }
}
